@@ -34,6 +34,8 @@ import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.OP;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.OPERATION_HEADERS;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.OP_ADDR;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.REQUIRED;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.RESOURCE_ADDED_NOTIFICATION;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.RESOURCE_REMOVED_NOTIFICATION;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.SERVER_CONFIG;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.SERVER_GROUP;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.USER;
@@ -68,6 +70,8 @@ import org.jboss.as.controller.client.OperationAttachments;
 import org.jboss.as.controller.client.OperationMessageHandler;
 import org.jboss.as.controller.descriptions.DescriptionProvider;
 import org.jboss.as.controller.descriptions.ModelDescriptionConstants;
+import org.jboss.as.controller.notification.Notification;
+import org.jboss.as.controller.notification.NotificationSupport;
 import org.jboss.as.controller.operations.global.GlobalOperationHandlers;
 import org.jboss.as.controller.operations.global.ReadResourceHandler;
 import org.jboss.as.controller.persistence.ConfigurationPersistenceException;
@@ -150,6 +154,7 @@ final class OperationContextImpl extends AbstractOperationContext {
     /** The step that acquired the container monitor  */
     private Step containerMonitorStep;
     private volatile Boolean requiresModelUpdateAuthorization;
+    private volatile boolean readOnly = true;
 
     /**
      * Cache of resource descriptions generated during operation execution. Primarily intended for
@@ -166,8 +171,9 @@ final class OperationContextImpl extends AbstractOperationContext {
                             final OperationMessageHandler messageHandler, final OperationAttachments attachments,
                             final Resource model, final ModelController.OperationTransactionControl transactionControl,
                             final ControlledProcessState processState, final AuditLogger auditLogger, final boolean booting,
-                            final Integer operationId, final HostServerGroupTracker hostServerGroupTracker) {
-        super(processType, runningMode, transactionControl, processState, booting, auditLogger);
+                            final Integer operationId, final HostServerGroupTracker hostServerGroupTracker,
+                            final NotificationSupport notificationSupport) {
+        super(processType, runningMode, transactionControl, processState, booting, auditLogger, notificationSupport);
         this.model = model;
         this.originalModel = model;
         this.modelController = modelController;
@@ -235,8 +241,11 @@ final class OperationContextImpl extends AbstractOperationContext {
 
 
     public ManagementResourceRegistration getResourceRegistrationForUpdate() {
-        final PathAddress address = activeStep.address;
         assert isControllingThread();
+
+        readOnly = false;
+
+        final PathAddress address = activeStep.address;
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -275,6 +284,11 @@ final class OperationContextImpl extends AbstractOperationContext {
 
     public ServiceRegistry getServiceRegistry(final boolean modify) throws UnsupportedOperationException {
         assert isControllingThread();
+
+        if (modify) {
+            readOnly = false;
+        }
+
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -294,6 +308,9 @@ final class OperationContextImpl extends AbstractOperationContext {
 
     public ServiceController<?> removeService(final ServiceName name) throws UnsupportedOperationException {
         assert isControllingThread();
+
+        readOnly = false;
+
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -342,6 +359,9 @@ final class OperationContextImpl extends AbstractOperationContext {
 
     public void removeService(final ServiceController<?> controller) throws UnsupportedOperationException {
         assert isControllingThread();
+
+        readOnly = false;
+
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -392,6 +412,9 @@ final class OperationContextImpl extends AbstractOperationContext {
 
     public ServiceTarget getServiceTarget() throws UnsupportedOperationException {
         assert isControllingThread();
+
+        readOnly = false;
+
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -464,8 +487,11 @@ final class OperationContextImpl extends AbstractOperationContext {
     }
 
     public ModelNode readModelForUpdate(final PathAddress requestAddress) {
-        final PathAddress address = activeStep.address.append(requestAddress);
         assert isControllingThread();
+
+        readOnly = false;
+
+        final PathAddress address = activeStep.address.append(requestAddress);
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -584,8 +610,11 @@ final class OperationContextImpl extends AbstractOperationContext {
     }
 
     public Resource readResourceForUpdate(PathAddress requestAddress) {
-        final PathAddress address = activeStep.address.append(requestAddress);
         assert isControllingThread();
+
+        readOnly = false;
+
+        final PathAddress address = activeStep.address.append(requestAddress);
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -593,12 +622,22 @@ final class OperationContextImpl extends AbstractOperationContext {
         if (currentStage != Stage.MODEL) {
             throw MESSAGES.stageAlreadyComplete(Stage.MODEL);
         }
-        rejectUserDomainServerUpdates();
+
+        // WFLY-3017 See if this write means a persistent config change
+        // For speed, we assume all calls during boot relate to persistent config
+        boolean runtimeOnly = !isBooting() && isResourceRuntimeOnly(address);
+
+        if (!runtimeOnly) {
+            rejectUserDomainServerUpdates();
+        }
         checkHostServerGroupTracker(address);
-        authorize(false, READ_WRITE_CONFIG);
-        if (!isModelAffected()) {
+        authorize(false, runtimeOnly ? READ_WRITE_RUNTIME : READ_WRITE_CONFIG);
+        if ((!runtimeOnly && !isModelAffected()) || (runtimeOnly && !affectsRuntime)) {
             takeWriteLock();
             model = model.clone();
+            if (runtimeOnly) {
+                affectsRuntime = true;
+            }
         }
         affectsModel.put(address, NULL);
         Resource resource = this.model;
@@ -611,21 +650,45 @@ final class OperationContextImpl extends AbstractOperationContext {
         return resource;
     }
 
+    private boolean isResourceRuntimeOnly(PathAddress fullAddress) {
+        Resource resource = this.model;
+        for (Iterator<PathElement> it = fullAddress.iterator(); it.hasNext() && resource != null;) {
+            PathElement element = it.next();
+            if (element.isMultiTarget()) {
+                resource = null;
+            } else {
+                resource = resource.getChild(element);
+            }
+        }
+
+        if (resource != null) {
+            return resource.isRuntime();
+        }
+        // No resource -- op will eventually fail
+        ImmutableManagementResourceRegistration mrr = modelController.getRootRegistration().getSubModel(fullAddress);
+        return mrr != null && mrr.isRuntimeOnly();
+    }
+
     @Override
     public Resource getOriginalRootResource() {
         // TODO restrict
         return originalModel.clone();
     }
 
+    @Override
     public Resource createResource(PathAddress relativeAddress) {
         final Resource toAdd = Resource.Factory.create();
         addResource(relativeAddress, toAdd);
         return toAdd;
     }
 
+    @Override
     public void addResource(PathAddress relativeAddress, Resource toAdd) {
-        final PathAddress absoluteAddress = activeStep.address.append(relativeAddress);
         assert isControllingThread();
+
+        readOnly = false;
+
+        final PathAddress absoluteAddress = activeStep.address.append(relativeAddress);
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -636,13 +699,21 @@ final class OperationContextImpl extends AbstractOperationContext {
         if (absoluteAddress.size() == 0) {
             throw MESSAGES.duplicateResourceAddress(absoluteAddress);
         }
-        // Check for user updates to a domain server model
-        rejectUserDomainServerUpdates();
+
+        boolean runtimeOnly = toAdd.isRuntime();
+
+        if (!runtimeOnly) {
+            // Check for user updates to a domain server model
+            rejectUserDomainServerUpdates();
+        }
         checkHostServerGroupTracker(absoluteAddress);
-        authorizeAdd();
-        if (!isModelAffected()) {
+        authorizeAdd(runtimeOnly);
+        if ((!runtimeOnly && !isModelAffected()) || (runtimeOnly && !affectsRuntime)) {
             takeWriteLock();
             model = model.clone();
+            if (runtimeOnly) {
+                affectsRuntime = true;
+            }
         }
         affectsModel.put(absoluteAddress, NULL);
         Resource model = this.model;
@@ -679,11 +750,18 @@ final class OperationContextImpl extends AbstractOperationContext {
                 }
             }
         }
+
+        Notification notification = new Notification(RESOURCE_ADDED_NOTIFICATION, absoluteAddress, ControllerMessages.MESSAGES.resourceWasAdded(absoluteAddress));
+        emit(notification);
     }
 
+    @Override
     public Resource removeResource(final PathAddress requestAddress) {
-        final PathAddress address = activeStep.address.append(requestAddress);
         assert isControllingThread();
+
+        readOnly = false;
+
+        final PathAddress address = activeStep.address.append(requestAddress);
         Stage currentStage = this.currentStage;
         if (currentStage == null) {
             throw MESSAGES.operationAlreadyComplete();
@@ -691,12 +769,21 @@ final class OperationContextImpl extends AbstractOperationContext {
         if (currentStage != Stage.MODEL) {
             throw MESSAGES.stageAlreadyComplete(Stage.MODEL);
         }
-        rejectUserDomainServerUpdates();
+
+        // WFLY-3017 See if this write means a persistent config change
+        // For speed, we assume all calls during boot relate to persistent config
+        boolean runtimeOnly = isResourceRuntimeOnly(address);
+        if (runtimeOnly) {
+            rejectUserDomainServerUpdates();
+        }
         checkHostServerGroupTracker(address);
-        authorize(false, READ_WRITE_CONFIG);
-        if (!isModelAffected()) {
+        authorize(false, runtimeOnly ? READ_WRITE_RUNTIME : READ_WRITE_CONFIG);
+        if ((!runtimeOnly && !isModelAffected()) || (runtimeOnly && !affectsRuntime)) {
             takeWriteLock();
             model = model.clone();
+            if (runtimeOnly) {
+                affectsRuntime = true;
+            }
         }
         affectsModel.put(address, NULL);
         Resource model = this.model;
@@ -712,13 +799,19 @@ final class OperationContextImpl extends AbstractOperationContext {
                 model = requireChild(model, element, address);
             }
         }
+
+        Notification notification = new Notification(RESOURCE_REMOVED_NOTIFICATION, address, ControllerMessages.MESSAGES.resourceWasRemoved(address));
+        emit(notification);
+
         return model;
     }
 
+    @Override
     public void acquireControllerLock() {
         takeWriteLock();
     }
 
+    @Override
     public Resource getRootResource() {
         // TODO limit children
         authorize(false, READ_CONFIG);
@@ -726,22 +819,27 @@ final class OperationContextImpl extends AbstractOperationContext {
         return readOnlyModel.clone();
     }
 
+    @Override
     public boolean isModelAffected() {
         return affectsModel.size() > 0;
     }
 
+    @Override
     public boolean isRuntimeAffected() {
         return affectsRuntime;
     }
 
+    @Override
     public boolean isResourceRegistryAffected() {
         return affectsResourceRegistration;
     }
 
+    @Override
     public Stage getCurrentStage() {
         return currentStage;
     }
 
+    @Override
     public void report(final MessageSeverity severity, final String message) {
         try {
             if(messageHandler != null) {
@@ -782,6 +880,11 @@ final class OperationContextImpl extends AbstractOperationContext {
         }
     }
 
+    @Override
+    boolean isReadOnly() {
+        return readOnly;
+    }
+
     private static Resource requireChild(final Resource resource, final PathElement childPath, final PathAddress fullAddress) {
         if (resource.hasChild(childPath)) {
             return resource.requireChild(childPath);
@@ -933,6 +1036,7 @@ final class OperationContextImpl extends AbstractOperationContext {
         return authResp;
     }
 
+    @Override
     Resource getModel() {
         return model;
     }
@@ -982,7 +1086,7 @@ final class OperationContextImpl extends AbstractOperationContext {
                         authResp = authorizeResource(true, false);
                         authResp.addOperationResult(operationName, authResult);
                     }
-                    authResult = authResp.validateAddAttributeEffects(ADD, targetAction.getActionEffects());
+                    authResult = authResp.validateAddAttributeEffects(ADD, targetAction.getActionEffects(), activeStep.operation);
                 }
             }
 
@@ -1027,18 +1131,19 @@ final class OperationContextImpl extends AbstractOperationContext {
         }
     }
 
-    private void authorizeAdd() {
+    private void authorizeAdd(boolean runtimeOnly) {
         AuthorizationResult accessResult = authorize(activeStep.operationId, activeStep.operation, false, ADDRESS);
         if (accessResult.getDecision() == AuthorizationResult.Decision.DENY) {
             throw ControllerMessages.MESSAGES.managementResourceNotFound(activeStep.address);
         }
-        AuthorizationResult authResult = authorize(activeStep.operationId, activeStep.operation, true, WRITE_CONFIG);
+        final Set<Action.ActionEffect> writeEffect = runtimeOnly ? WRITE_RUNTIME : WRITE_CONFIG;
+        AuthorizationResult authResult = authorize(activeStep.operationId, activeStep.operation, true, writeEffect);
         if (authResult.getDecision() == AuthorizationResult.Decision.DENY) {
             AuthorizationResponseImpl authResp = authorizations.get(activeStep.operationId);
             assert authResp != null : "no AuthorizationResponse";
             String opName = activeStep.operation.get(OP).asString();
             authResp.addOperationResult(opName, authResult);
-            authResult = authResp.validateAddAttributeEffects(opName, WRITE_CONFIG);
+            authResult = authResp.validateAddAttributeEffects(opName, writeEffect, activeStep.operation);
             authResp.addOperationResult(opName, authResult);
             if (authResult.getDecision() == AuthorizationResult.Decision.DENY) {
                 throw ControllerMessages.MESSAGES.unauthorized(activeStep.operationId.name, activeStep.address, authResult.getExplanation());
@@ -1070,21 +1175,25 @@ final class OperationContextImpl extends AbstractOperationContext {
                     return effectResult;
                 }
             }
+            AuthorizationResult errResult = null;
+
             if (allAttributes) {
                 ImmutableManagementResourceRegistration mrr = authResp.targetResource.getResourceRegistration();
                 ModelNode model = authResp.targetResource.getResource().getModel();
                 Set<Action.ActionEffect> attributeEffects = actionEffects.isEmpty() ? authResp.standardAction.getActionEffects() : actionEffects;
+
                 for (String attr : mrr.getAttributeNames(PathAddress.EMPTY_ADDRESS)) {
                     ModelNode currentValue = model.has(attr) ? model.get(attr) : new ModelNode();
                     AuthorizationResult attrResult = authorize(opId, attr, currentValue, attributeEffects);
                     if (attrResult.getDecision() == AuthorizationResult.Decision.DENY) {
-                        return attrResult;
+
+                        errResult = attrResult;
                     }
                 }
                 authResp.attributesComplete = true;
             }
 
-            return AuthorizationResult.PERMITTED;
+            return errResult != null ? errResult : AuthorizationResult.PERMITTED;
         }
     }
 
@@ -1645,7 +1754,7 @@ final class OperationContextImpl extends AbstractOperationContext {
             operationResults.put(operationName, result);
         }
 
-        private AuthorizationResult validateAddAttributeEffects(String operationName, Set<ActionEffect> actionEffects) {
+        private AuthorizationResult validateAddAttributeEffects(String operationName, Set<ActionEffect> actionEffects, ModelNode operation) {
             AuthorizationResult basic = operationResults.get(operationName);
             assert basic != null : " no basic authorization has been performed for operation 'add'";
             if (basic.getDecision() == Decision.DENY) {
@@ -1658,7 +1767,11 @@ final class OperationContextImpl extends AbstractOperationContext {
                     if (attrDenied) {
                         attributeWasDenied = true;
                         // See if we even care about this attribute
-                        if (model.hasDefined(attrName) || isAddableAttribute(attrName, resourceRegistration)) {
+                        // BES 2014/11/11 -- currently "model" is always undefined at this point, so testing it is silly,
+                        // but I leave it in here as a 2nd line of defense in case we rework something and this check
+                        // gets run after model gets populated
+                        if (operation.hasDefined(attrName) || model.hasDefined(attrName) ||
+                                isAddableAttribute(attrName, resourceRegistration)) {
                             Map<ActionEffect, AuthorizationResult> attrResults = attributeResultEntry.getValue();
                             for (ActionEffect actionEffect : actionEffects) {
                                 AuthorizationResult authResult = attrResults.get(actionEffect);
