@@ -22,10 +22,14 @@
 package org.jboss.as.controller.test;
 
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.ATTRIBUTES_ONLY;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.ATTRIBUTE_VALUE_WRITTEN_NOTIFICATION;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.CHILD_TYPE;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.INCLUDE_DEFAULTS;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.INCLUDE_RUNTIME;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.INHERITED;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.NAME;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.NOTIFICATIONS;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.NOTIFICATION_TYPE;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.OPERATIONS;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.OPERATION_NAME;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.OP_ADDR;
@@ -39,6 +43,8 @@ import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.REA
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.READ_RESOURCE_OPERATION;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.RECURSIVE;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.REQUEST_PROPERTIES;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.RESOURCE_ADDED_NOTIFICATION;
+import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.RESOURCE_REMOVED_NOTIFICATION;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.RUNTIME_ONLY;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.TYPE;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.VALUE;
@@ -56,6 +62,7 @@ import java.util.List;
 import java.util.Set;
 
 import org.jboss.as.controller.OperationFailedException;
+import org.jboss.as.controller.descriptions.ModelDescriptionConstants;
 import org.jboss.dmr.ModelNode;
 import org.jboss.dmr.ModelType;
 import org.jboss.dmr.Property;
@@ -234,10 +241,10 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         assertEquals("l1A", list.get(0).asString());
         assertEquals("l1B", list.get(1).asString());
 
-        operation.get(NAME).set("non-existant-attribute");
+        operation.get(NAME).set("non-existent-attribute");
         try {
             result = executeForResult(operation);
-            fail("Expected error for non-existant attribute");
+            fail("Expected error for non-existent attribute");
         } catch (OperationFailedException expected) {
         }
 
@@ -332,16 +339,16 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         assertTrue(names.contains("other"));
 
 
-        operation.get(CHILD_TYPE).set("non-existant-child");
+        operation.get(CHILD_TYPE).set("non-existent-child");
         try {
             result = executeForResult(operation);
-            fail("Expected error for non-existant child");
+            fail("Expected error for non-existent child");
         } catch (OperationFailedException expected) {
         }
 
         operation = createOperation(READ_CHILDREN_NAMES_OPERATION, "profile", "profileC", "subsystem", "subsystem4");
         operation.get(CHILD_TYPE).set("type1");
-        // BES 2011/06/06 These assertions make no sense; as they are no different from "non-existant-child" case
+        // BES 2011/06/06 These assertions make no sense; as they are no different from "non-existent-child" case
         // Replacing with a fail check
 //          result = executeForResult(operation);
 //        assertNotNull(result);
@@ -386,10 +393,10 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         assertTrue(Arrays.asList("type1", "type2").containsAll(stringList));
 
 
-        operation = createOperation(READ_CHILDREN_TYPES_OPERATION, "profile", "profileA", "subsystem", "non-existant");
+        operation = createOperation(READ_CHILDREN_TYPES_OPERATION, "profile", "profileA", "subsystem", "non-existent");
         try {
             result = executeForResult(operation);
-            fail("Expected error for non-existant child");
+            fail("Expected error for non-existent child");
         } catch (OperationFailedException expected) {
         }
 
@@ -446,16 +453,16 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         assertNotNull(other);
         assertEquals("Name2", other.require(NAME).asString());
 
-        operation.get(CHILD_TYPE).set("non-existant-child");
+        operation.get(CHILD_TYPE).set("non-existent-child");
         try {
             result = executeForResult(operation);
-            fail("Expected error for non-existant child");
+            fail("Expected error for non-existent child");
         } catch (OperationFailedException expected) {
         }
 
         operation = createOperation(READ_CHILDREN_RESOURCES_OPERATION, "profile", "profileC", "subsystem", "subsystem4");
         operation.get(CHILD_TYPE).set("type1");
-        // BES 2011/06/06 These assertions make no sense; as they are no different from "non-existant-child" case
+        // BES 2011/06/06 These assertions make no sense; as they are no different from "non-existent-child" case
         // Replacing with a fail check
 //          result = executeForResult(operation);
 //        assertNotNull(result);
@@ -514,16 +521,16 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         assertNotNull(other);
         assertEquals("Name2", other.require(NAME).asString());
 
-        operation.get(CHILD_TYPE).set("non-existant-child");
+        operation.get(CHILD_TYPE).set("non-existent-child");
         try {
             result = executeForResult(operation);
-            fail("Expected error for non-existant child");
+            fail("Expected error for non-existent child");
         } catch (OperationFailedException expected) {
         }
 
         operation = createOperation(READ_CHILDREN_RESOURCES_OPERATION, "profile", "profileC", "subsystem", "subsystem4");
         operation.get(CHILD_TYPE).set("type1");
-        // BES 2011/06/06 These assertions make no sense; as they are no different from "non-existant-child" case
+        // BES 2011/06/06 These assertions make no sense; as they are no different from "non-existent-child" case
         // Replacing with a fail check
 //          result = executeForResult(operation);
 //        assertNotNull(result);
@@ -625,12 +632,12 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
     public void testReadResourceDescriptionOperation() throws Exception {
         ModelNode operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION);
         ModelNode result = executeForResult(operation);
-        checkRootNodeDescription(result, false, false);
+        checkRootNodeDescription(result, false, false, false);
         assertFalse(result.get(OPERATIONS).isDefined());
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA");
         result = executeForResult(operation);
-        checkProfileNodeDescription(result, false, false);
+        checkProfileNodeDescription(result, false, false, false);
 
         //TODO this is not possible - the wildcard address does not correspond to anything in the real model
         //operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "*");
@@ -639,7 +646,7 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
         result = executeForResult(operation);
-        checkSubsystem1Description(result, false, false);
+        checkSubsystem1Description(result, false, false, false);
         assertFalse(result.get(OPERATIONS).isDefined());
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing1");
@@ -663,13 +670,13 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         ModelNode operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION);
         operation.get(RECURSIVE).set(true);
         ModelNode result = executeForResult(operation);
-        checkRootNodeDescription(result, true, false);
+        checkRootNodeDescription(result, true, false, false);
         assertFalse(result.get(OPERATIONS).isDefined());
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA");
         operation.get(RECURSIVE).set(true);
         result = executeForResult(operation);
-        checkProfileNodeDescription(result, true, false);
+        checkProfileNodeDescription(result, true, false, false);
 
         //TODO this is not possible - the wildcard address does not correspond to anything in the real model
         //operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "*");
@@ -680,7 +687,7 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
         operation.get(RECURSIVE).set(true);
         result = executeForResult(operation);
-        checkSubsystem1Description(result, true, false);
+        checkSubsystem1Description(result, true, false, false);
         assertFalse(result.get(OPERATIONS).isDefined());
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing1");
@@ -707,7 +714,7 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         ModelNode operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION);
         operation.get(OPERATIONS).set(true);
         ModelNode result = executeForResult(operation);
-        checkRootNodeDescription(result, false, true);
+        checkRootNodeDescription(result, false, true, false);
         assertTrue(result.require(OPERATIONS).isDefined());
         Set<String> ops = result.require(OPERATIONS).keys();
         assertTrue(ops.contains(READ_ATTRIBUTE_OPERATION));
@@ -724,7 +731,7 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
         operation.get(OPERATIONS).set(true);
         result = executeForResult(operation);
-        checkSubsystem1Description(result, false, true);
+        checkSubsystem1Description(result, false, true, false);
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing1");
         operation.get(OPERATIONS).set(true);
@@ -748,14 +755,14 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         operation.get(OPERATIONS).set(true);
         operation.get(RECURSIVE).set(true);
         ModelNode result = executeForResult(operation);
-        checkRootNodeDescription(result, true, true);
+        checkRootNodeDescription(result, true, true, false);
 
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
         operation.get(OPERATIONS).set(true);
         operation.get(RECURSIVE).set(true);
         result = executeForResult(operation);
-        checkSubsystem1Description(result, true, true);
+        checkSubsystem1Description(result, true, true, false);
 
         operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing1");
         operation.get(OPERATIONS).set(true);
@@ -776,6 +783,108 @@ public class GlobalOperationsTestCase extends AbstractGlobalOperationsTestCase {
         checkType2Description(result);
     }
 
+    @Test
+    public void testReadResourceDescriptionOperationWithNotifications() throws Exception {
+        ModelNode operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION);
+        operation.get(NOTIFICATIONS).set(true);
+        ModelNode result = executeForResult(operation);
+        checkRootNodeDescription(result, false, false, true);
+
+        assertTrue(result.require(NOTIFICATIONS).isDefined());
+        Set<String> notifs = result.require(NOTIFICATIONS).keys();
+        assertTrue(notifs.contains(RESOURCE_ADDED_NOTIFICATION));
+        assertTrue(notifs.contains(RESOURCE_REMOVED_NOTIFICATION));
+        assertTrue(notifs.contains(ATTRIBUTE_VALUE_WRITTEN_NOTIFICATION));
+        for (String notif : notifs) {
+            assertEquals(notif, result.require(NOTIFICATIONS).require(notif).require(NOTIFICATION_TYPE).asString());
+        }
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
+        operation.get(NOTIFICATIONS).set(true);
+        result = executeForResult(operation);
+        checkSubsystem1Description(result, false, false, true);
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing1");
+        operation.get(NOTIFICATIONS).set(true);
+        result = executeForResult(operation);
+        checkType1Description(result);
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing2");
+        operation.get(NOTIFICATIONS).set(true);
+        result = executeForResult(operation);
+        checkType1Description(result);
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type2", "other");
+        operation.get(NOTIFICATIONS).set(true);
+        result = executeForResult(operation);
+        checkType2Description(result);
+    }
+
+    @Test
+    public void testReadResourceDescriptionOperationWithNotInheritedNotifications() throws Exception {
+        ModelNode operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION);
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(INHERITED).set(false);
+        ModelNode result = executeForResult(operation);
+        checkRootNodeDescription(result, false, false, true);
+
+        assertTrue(result.require(NOTIFICATIONS).isDefined());
+        Set<String> notifs = result.require(NOTIFICATIONS).keys();
+        assertTrue(notifs.contains(RESOURCE_ADDED_NOTIFICATION));
+        assertTrue(notifs.contains(RESOURCE_REMOVED_NOTIFICATION));
+        assertTrue(notifs.contains(ATTRIBUTE_VALUE_WRITTEN_NOTIFICATION));
+        for (String notif : notifs) {
+            assertEquals(notif, result.require(NOTIFICATIONS).require(notif).require(NOTIFICATION_TYPE).asString());
+        }
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(INHERITED).set(false);
+        result = executeForResult(operation);
+        checkSubsystem1Description(result, false, false, false);
+    }
+
+    @Test
+    public void testRecursiveReadResourceDescriptionOperationWithNotifications() throws Exception {
+        ModelNode operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION);
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(RECURSIVE).set(true);
+        ModelNode result = executeForResult(operation);
+        checkRootNodeDescription(result, true, false, true);
+
+        assertTrue(result.require(NOTIFICATIONS).isDefined());
+        Set<String> notifs = result.require(NOTIFICATIONS).keys();
+        assertTrue(notifs.contains(RESOURCE_ADDED_NOTIFICATION));
+        assertTrue(notifs.contains(RESOURCE_REMOVED_NOTIFICATION));
+        assertTrue(notifs.contains(ATTRIBUTE_VALUE_WRITTEN_NOTIFICATION));
+        for (String notif : notifs) {
+            assertEquals(notif, result.require(NOTIFICATIONS).require(notif).require(NOTIFICATION_TYPE).asString());
+        }
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1");
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(RECURSIVE).set(true);
+        result = executeForResult(operation);
+        checkSubsystem1Description(result, true, false, true);
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing1");
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(RECURSIVE).set(true);
+        result = executeForResult(operation);
+        checkType1Description(result);
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type1", "thing2");
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(RECURSIVE).set(true);
+        result = executeForResult(operation);
+        checkType1Description(result);
+
+        operation = createOperation(READ_RESOURCE_DESCRIPTION_OPERATION, "profile", "profileA", "subsystem", "subsystem1", "type2", "other");
+        operation.get(NOTIFICATIONS).set(true);
+        operation.get(RECURSIVE).set(true);
+        result = executeForResult(operation);
+        checkType2Description(result);
+    }
+
     @Test
     public void testReadResourceAttributesOnly() throws Exception {
         ModelNode operation = createOperation(READ_RESOURCE_OPERATION);
