@@ -27,6 +27,7 @@ import java.util.Collection;
 import java.util.EnumSet;
 import java.util.logging.Handler;
 
+import org.jboss.as.controller.NoSuchResourceException;
 import org.jboss.as.controller.OperationFailedException;
 import org.jboss.as.server.deployment.DeploymentUnitProcessingException;
 import org.jboss.logging.Messages;
@@ -267,7 +268,7 @@ public interface LoggingMessages {
     String invalidPath(String relativeTo, String path);
 
     /**
-     * A message indicating an suffix is invalid.
+     * A message indicating a suffix is invalid.
      *
      * @param suffix the suffix.
      *
@@ -611,7 +612,7 @@ public interface LoggingMessages {
     IllegalArgumentException truncatedFilterExpression();
 
     /**
-     * Creates an exception indicating the an invalid escape was found in the filter expression string.
+     * Creates an exception indicating an invalid escape was found in the filter expression string.
      *
      * @return an {@link IllegalArgumentException} for the error
      */
@@ -701,7 +702,7 @@ public interface LoggingMessages {
      * @return an {@link OperationFailedException} for the error
      */
     @Message(id = 11593, value = "Failed to read the log file '%s'")
-    OperationFailedException failedToReadLogFile(@Cause Throwable cause, String name);
+    RuntimeException failedToReadLogFile(@Cause Throwable cause, String name);
 
     /**
      * Creates an exception indicating the file was found in the log directory.
@@ -712,7 +713,7 @@ public interface LoggingMessages {
      * @return an {@link OperationFailedException} for the error
      */
     @Message(id = 11594, value = "File '%s' was not found and cannot be found in the %s directory property.")
-    OperationFailedException logFileNotFound(String name, String directoryProperty);
+    NoSuchResourceException logFileNotFound(String name, String directoryProperty);
 
     /**
      * Creates an exception indicating the user cannot read the file.
@@ -723,4 +724,20 @@ public interface LoggingMessages {
      */
     @Message(id = 11595, value = "File '%s' is not allowed to be read.")
     OperationFailedException readNotAllowed(String name);
+
+    /**
+     * Create a failure description message indicating that the resource of given type can not be registered.
+     *
+     * @return an {@link UnsupportedOperationException} for the error
+     */
+    @Message(id = 11599, value = "Resources of type %s cannot be registered")
+    UnsupportedOperationException cannotRegisterResourceOfType(String childType);
+
+    /**
+     * Create a failure description message indicating that the resource of given type can not be removed.
+     *
+     * @return an {@link UnsupportedOperationException} for the error
+     */
+    @Message(id = 15929, value = "Resources of type %s cannot be removed")
+    UnsupportedOperationException cannotRemoveResourceOfType(String childType);
 }
