@@ -40,6 +40,7 @@ import org.hornetq.api.core.DiscoveryGroupConfiguration;
 import org.hornetq.api.core.TransportConfiguration;
 import org.hornetq.core.config.Configuration;
 import org.hornetq.core.journal.impl.AIOSequentialFileFactory;
+import org.hornetq.core.remoting.impl.netty.TransportConstants;
 import org.hornetq.core.server.HornetQServer;
 import org.hornetq.core.server.JournalType;
 import org.hornetq.core.server.impl.HornetQServerImpl;
@@ -49,6 +50,7 @@ import org.jboss.as.controller.PathAddress;
 import org.jboss.as.controller.descriptions.ModelDescriptionConstants;
 import org.jboss.as.controller.services.path.AbsolutePathService;
 import org.jboss.as.controller.services.path.PathManager;
+import org.jboss.as.network.NetworkUtils;
 import org.jboss.as.network.OutboundSocketBinding;
 import org.jboss.as.network.SocketBinding;
 import org.jboss.as.security.plugins.SecurityDomainContext;
@@ -180,18 +182,21 @@ class HornetQService implements Service<HornetQServer> {
                             }
                             InetSocketAddress sa = socketBinding.getSocketAddress();
                             port = sa.getPort();
-                            // resolve the host name of the address only if a loopback adress has been set
+                            // resolve the host name of the address only if a loopback address has been set
                             if (sa.getAddress().isLoopbackAddress()) {
-                                host = sa.getAddress().getHostName();
+                                host = NetworkUtils.canonize(sa.getAddress().getHostName());
                             } else {
-                                host = sa.getAddress().getHostAddress();
+                                host = NetworkUtils.canonize(sa.getAddress().getHostAddress());
                             }
                         } else {
                             port = binding.getDestinationPort();
-                            if (binding.getDestinationAddress().isLoopbackAddress()) {
-                                host = binding.getDestinationAddress().getHostName();
-                            } else {
-                                host = binding.getDestinationAddress().getHostAddress();
+                            host = NetworkUtils.canonize(binding.getUnresolvedDestinationAddress());
+                            if (binding.getSourceAddress() != null) {
+                                tc.getParams().put(TransportConstants.LOCAL_ADDRESS_PROP_NAME,
+                                        NetworkUtils.canonize(binding.getSourceAddress().getHostAddress()));
+                            }
+                            if (binding.getSourcePort() != null) {
+                                tc.getParams().put(TransportConstants.LOCAL_PORT_PROP_NAME, binding.getSourcePort());
                             }
                         }
                         tc.getParams().put(HOST, host);
