@@ -343,6 +343,11 @@ public class ServerControllerUnitTestCase {
             delegate.registerAttributes(resourceRegistration);
         }
 
+        @Override
+        public void registerNotifications(ManagementResourceRegistration resourceRegistration) {
+            delegate.registerNotifications(resourceRegistration);
+        }
+
         @Override
         public PathElement getPathElement() {
             return delegate.getPathElement();
